@@ -31,6 +31,7 @@ from strongarm.macho.macho_definitions import (
     DylibStruct,
     LcStrUnion,
     MachArch,
+    MachoBuildTool,
     MachoBuildVersionPlatform,
     MachoFileType,
     StaticFilePointer,
@@ -113,6 +114,14 @@ class MachoSection:
         return f'<MachoSection {virtual_loc} "{self.name}" ("{self.segment_name}")>'
 
 
+def _version_from_nibbles(value: int) -> LooseVersion:
+    # X.Y.Z is encoded in nibbles xxxx.yy.zz
+    patch = (value >> (8 * 0)) & 0xFF
+    minor = (value >> (8 * 1)) & 0xFF
+    major = (value >> (8 * 2)) & 0xFFFF
+    return LooseVersion(f"{major}.{minor}.{patch}")
+
+
 class MachoBinary:
     _MAG_64 = [MachArch.MH_MAGIC_64, MachArch.MH_CIGAM_64]
     _MAG_32 = [MachArch.MH_MAGIC, MachArch.MH_CIGAM]
@@ -160,6 +169,8 @@ class MachoBinary:
 
         self.__codesign_parser: Optional[CodesignParser] = None
         self.__minimum_deployment_target: Optional[LooseVersion] = None
+        self.__sdk_deployment_target: Optional[LooseVersion] = None
+        self.__build_tools: Dict[str, LooseVersion] = {}
 
         # This kicks off the parse of the binary
         if not self.parse():
@@ -902,11 +913,11 @@ class MachoBinary:
         # TODO(PT): Alternatively, assignments like `self.header.ncmds += 1` should update the backing binary.
         # This would also make binary modifications easier.
 
-        # Check that there is enough emtpy space before the start of __text to insert a load command
+        # Check that there is enough emtpy space before the start of the first section to insert a load command
         load_commands_end = self.header.sizeof + self.header.sizeofcmds
         string_end = load_commands_end + load_cmd.cmdsize
-        text_section = self.section_with_name("__text", "__TEXT")
-        if text_section and string_end >= text_section.offset:
+        first_section = sorted(self.sections, key=lambda s: s.address)[0]
+        if string_end >= first_section.offset:
             raise NoEmptySpaceForLoadCommandError()
 
         # Add the load command to the end of the Mach-O header
@@ -1062,16 +1073,15 @@ class MachoBinary:
         return dylib_name
 
     def get_minimum_deployment_target(self) -> Optional[LooseVersion]:
-        if not self.__minimum_deployment_target:
-            if self._build_version_cmd:
-                # X.Y.Z is encoded in nibbles xxxx.yy.zz
-                encoded_min_target = self._build_version_cmd.minos
-                patch = (encoded_min_target >> (8 * 0)) & 0xFF
-                minor = (encoded_min_target >> (8 * 1)) & 0xFF
-                major = (encoded_min_target >> (8 * 2)) & 0xFFFF
-                self.__minimum_deployment_target = LooseVersion(f"{major}.{minor}.{patch}")
+        if not self.__minimum_deployment_target and self._build_version_cmd:
+            self.__minimum_deployment_target = _version_from_nibbles(self._build_version_cmd.minos)
         return self.__minimum_deployment_target
 
+    def get_sdk_deployment_target(self) -> Optional[LooseVersion]:
+        if not self.__sdk_deployment_target and self._build_version_cmd:
+            self.__sdk_deployment_target = _version_from_nibbles(self._build_version_cmd.sdk)
+        return self.__sdk_deployment_target
+
     def get_build_version_platform(self) -> Optional[MachoBuildVersionPlatform]:
         if not self._build_version_cmd:
             return None
@@ -1079,3 +1089,23 @@ class MachoBinary:
 
     def get_build_tool_versions(self) -> Optional[List[MachoBuildToolVersionStruct]]:
         return self._build_tool_versions
+
+    def get_build_tools(self) -> Dict[str, LooseVersion]:
+        if not self.__build_tools and self._build_tool_versions:
+            self.__build_tools = {
+                MachoBuildTool(tool.tool).name.lower(): _version_from_nibbles(tool.version)
+                for tool in self._build_tool_versions
+            }
+        return self.__build_tools
+
+    def get_cstring_section(self) -> Optional[MachoSection]:
+        # Ordered by likelihood
+        container_segments = [
+            "__TEXT",
+            "__RODATA",
+        ]
+        for segment in container_segments:
+            strings_section = self.section_with_name("__cstring", segment)
+            if strings_section:
+                return strings_section
+        return None
