@@ -1,10 +1,7 @@
-import logging
 from ctypes import c_uint32, c_uint64, sizeof
 from typing import Dict, List, Optional
 
-from more_itertools import first_true
-
-from strongarm.macho import DyldInfoParser
+from strongarm.logger import strongarm_logger
 from strongarm.macho.arch_independent_structs import (
     ArchIndependentStructure,
     ObjcCategoryRawStruct,
@@ -20,6 +17,8 @@ from strongarm.macho.arch_independent_structs import (
 from strongarm.macho.macho_binary import MachoBinary
 from strongarm.macho.macho_definitions import VirtualMemoryPointer
 
+logger = strongarm_logger.getChild(__file__)
+
 
 class ObjcSelref:
     __slots__ = ["source_address", "destination_address", "selector_literal"]
@@ -138,23 +137,25 @@ class ObjcCategory(ObjcClass):
 
 
 class ObjcRuntimeDataParser:
-    def __init__(self, binary: MachoBinary, dyld_info_parser: DyldInfoParser) -> None:
+    def __init__(self, binary: MachoBinary) -> None:
         self.binary = binary
-        logging.debug(f"Parsing ObjC runtime info of {self.binary}...")
+        logger.debug(f"Parsing ObjC runtime info of {self.binary}...")
 
-        logging.debug("Step 1: Parsing selrefs...")
-        self._selref_ptr_to_selector_map: Dict[VirtualMemoryPointer, ObjcSelector] = {}
+        logger.debug("Step 1: Parsing selrefs...")
         self._selector_literal_ptr_to_selref_map: Dict[VirtualMemoryPointer, ObjcSelref] = {}
-        # This populates self._selector_literal_ptr_to_selref_map and self._selref_ptr_to_selector_map
+        self._selref_ptr_to_selref_map: Dict[VirtualMemoryPointer, ObjcSelref] = {}
+        # Note this mapping is partially filled in now, but gets updated later in the parse
+        self._selref_ptr_to_selector_map: Dict[VirtualMemoryPointer, ObjcSelector] = {}
+        # Populates the mappings above
         self._parse_selrefs()
 
-        logging.debug("Step 2: Parsing classes, categories, and protocols...")
+        logger.debug("Step 2: Parsing classes, categories, and protocols...")
         self._classrefs_to_objc_classes: Dict[VirtualMemoryPointer, ObjcClass] = {}
         # This populates self._classrefs_to_objc_classes
-        self.classes = self._parse_class_and_category_info(dyld_info_parser)
+        self.classes = self._parse_class_and_category_info()
         self.protocols = self._parse_global_protocol_info()
 
-        logging.debug("Step 3: Resolving symbol name to source dylib map...")
+        logger.debug("Step 3: Resolving symbol name to source dylib map...")
         self._sym_to_dylib_path = self._parse_linked_dylib_symbols()
 
     def _parse_linked_dylib_symbols(self) -> Dict[str, str]:
@@ -163,15 +164,24 @@ class ObjcRuntimeDataParser:
         symtab = self.binary.symtab
         symtab_contents = self.binary.symtab_contents
         dysymtab = self.binary.dysymtab
+        visited_addresses = set()
         for undef_sym_idx in range(dysymtab.nundefsym):
             symtab_idx = dysymtab.iundefsym + undef_sym_idx
             sym = symtab_contents[symtab_idx]
 
             strtab_idx = sym.n_un.n_strx
             string_file_address = symtab.stroff + strtab_idx
+
+            # Some binaries contain a symtab such that all the calculated string address are the same. This check
+            # prevents spamming the logs with errors about the same symbol
+            # TODO(FS): Task tracking this issue SCAN-2744
+            if string_file_address in visited_addresses:
+                continue
+            visited_addresses.add(string_file_address)
+
             symbol_name = self.binary.get_full_string_from_start_address(string_file_address, virtual=False)
             if not symbol_name:
-                logging.error(f"Could not get symbol name at address {string_file_address}")
+                logger.error(f"Could not get symbol name at address {hex(string_file_address)}")
                 continue
 
             library_ordinal = self._library_ordinal_from_n_desc(sym.n_desc)
@@ -191,43 +201,45 @@ class ObjcRuntimeDataParser:
 
     def _parse_selrefs(self) -> None:
         """Parse the binary's selref list, and store the data.
-
-        This method populates self._selector_literal_ptr_to_selref_map.
-        It also *PARTLY* populates self._selref_ptr_to_selector_map. All selrefs keys will have an ObjcSelector
-        value, but none of the ObjcSelector objects will have their `implementation` field filled, because
-        at this point in the parse we do not yet know the implementations of each selector. ObjcSelectors which we
-        later find an implementation for are updated in self.read_selectors_from_methlist_ptr"""
-        selref_pointers, selector_literal_pointers = self.binary.read_pointer_section("__objc_selrefs")
-        # sanity check
-        if len(selref_pointers) != len(selector_literal_pointers):
-            raise RuntimeError("read invalid data from __objc_selrefs")
-
-        for selref_ptr, selector_literal_ptr in zip(selref_pointers, selector_literal_pointers):
-            # read selector string literal from selref pointer
+        Fully populates:
+            self._selector_literal_ptr_to_selref_map
+            self._selref_ptr_to_selref_map
+        *PARTIALLY* populates:
+            self._selref_ptr_to_selector_map
+        All selrefs keys will have an ObjcSelector value, but none of the ObjcSelector objects
+        will have their `implementation` field filled, because at this point in the parse we do not yet know the
+        implementations of each selector. ObjcSelectors which we later find an implementation for are
+        updated in self.read_selectors_from_methlist_ptr().
+        """
+        selref_pointers = self.binary.read_pointer_section("__objc_selrefs")
+        for selref_ptr, selector_literal_ptr in selref_pointers.items():
+            # Read selector string literal from selref pointer
             selector_string = self.binary.get_full_string_from_start_address(selector_literal_ptr)
             if not selector_string:
-                continue  # but all selectors should have a name
+                # But all selectors should have a name
+                # TODO(PT): Is this an error?
+                continue
             wrapped_selref = ObjcSelref(selref_ptr, selector_literal_ptr, selector_string)
 
-            # map the selector string pointer to the ObjcSelref
+            # Map the pointers to the wrapped-up selref object
             self._selector_literal_ptr_to_selref_map[selector_literal_ptr] = wrapped_selref
-            # add second mapping in selref list
-            # we don't know the implementation address yet but it will be updated when we parse method lists
+            self._selref_ptr_to_selref_map[wrapped_selref.source_address] = wrapped_selref
+
+            # And start off the selref pointer -> selector map
+            # We don't know the implementation address yet but it will be updated when we parse method lists
             self._selref_ptr_to_selector_map[selref_ptr] = ObjcSelector(selector_string, wrapped_selref, None)
 
     def selector_for_selref(self, selref_addr: VirtualMemoryPointer) -> Optional[ObjcSelector]:
+        # This map contains selectors implemented in the binary
         selector = self._selref_ptr_to_selector_map.get(selref_addr)
         if selector is not None:
             return selector
 
-        # selref wasn't referenced in classes implemented within the binary
-        # make sure it's a valid selref
-        selrefs = self._selector_literal_ptr_to_selref_map.values()
-        selref = first_true(iter(selrefs), pred=lambda x: x.source_address == selref_addr, default=None)
-
+        # This map contains all used selectors, including those imported from an external binary
+        selref = self._selref_ptr_to_selref_map.get(selref_addr)
         if selref is not None:
-            # Therefore, the selref must refer to a selector which is defined outside this binary
-            # this is fine, just construct an ObjcSelector with what we know
+            # The selref must refer to a selector which is defined outside this binary
+            # This is fine, just construct an ObjcSelector with what we know
             return ObjcSelector(selref.selector_literal, selref, None)
 
         else:
@@ -250,8 +262,7 @@ class ObjcRuntimeDataParser:
         )
 
     def get_method_imp_addresses(self, selector: str) -> List[VirtualMemoryPointer]:
-        """Given a selector, return a list of virtual addresses corresponding to the start of each IMP for that SEL
-        """
+        """Given a selector, return a list of virtual addresses corresponding to the start of each IMP for that SEL."""
         return [
             objc_sel.implementation
             for objc_class in self.classes
@@ -263,9 +274,8 @@ class ObjcRuntimeDataParser:
         return self._classrefs_to_objc_classes.get(classlist_ptr)
 
     def _parse_objc_classes(self) -> List[ObjcClass]:
-        """Read Objective-C class data in __objc_classlist, __objc_data to get classes and selectors in binary
-        """
-        logging.debug("Cross-referencing __objc_classlist, __objc_class, and __objc_data entries...")
+        """Read Objective-C class data in __objc_classlist, __objc_data to get classes and selectors in binary."""
+        logger.debug("Cross-referencing __objc_classlist, __objc_class, and __objc_data entries...")
         parsed_objc_classes = []
         classlist_pointers = self._get_classlist_pointers()
         for ptr in classlist_pointers:
@@ -303,7 +313,7 @@ class ObjcRuntimeDataParser:
         return parsed_objc_classes
 
     def _parse_objc_categories(self) -> List[ObjcCategory]:
-        logging.debug("Cross referencing __objc_catlist, __objc_category, and __objc_data entries...")
+        logger.debug("Cross referencing __objc_catlist, __objc_category, and __objc_data entries...")
         parsed_categories = []
         category_pointers = self._get_catlist_pointers()
         for ptr in category_pointers:
@@ -313,33 +323,16 @@ class ObjcRuntimeDataParser:
                 parsed_categories.append(parsed_category)
         return parsed_categories
 
-    def _parse_class_and_category_info(self, dyld_info_parser: DyldInfoParser) -> List[ObjcClass]:
-        """Parse classes and categories referenced by __objc_classlist and __objc_catlist
-        """
+    def _parse_class_and_category_info(self) -> List[ObjcClass]:
+        """Parse classes and categories referenced by __objc_classlist and __objc_catlist."""
         classes: List[ObjcClass] = []
         classes += self._parse_objc_classes()
         classes += self._parse_objc_categories()
-        # Link superclass methods into their subclasses
-        self._add_superclass_methods_to_subclasses()
         # Link superclasses of classes and base-classes of categories
-        self._add_superclass_or_base_class_name_to_classes(classes, dyld_info_parser)
+        self._add_superclass_or_base_class_name_to_classes(classes)
         return classes
 
-    def _add_superclass_methods_to_subclasses(self) -> None:
-        for classlist_ptr, objc_class in self._classrefs_to_objc_classes.items():
-            super_classref = objc_class.super_classref
-            if not super_classref:
-                continue
-            superclass = self.objc_class_for_classlist_pointer(super_classref)
-            if not superclass:
-                continue
-            objc_class.ivars += superclass.ivars
-            objc_class.selectors += superclass.selectors
-            objc_class.protocols += superclass.protocols
-
-    def _add_superclass_or_base_class_name_to_classes(
-        self, classes: List[ObjcClass], dyld_info_parser: DyldInfoParser
-    ) -> None:
+    def _add_superclass_or_base_class_name_to_classes(self, classes: List[ObjcClass]) -> None:
         """Iterate each ObjC class/category, and backfill its superclass/base_class name, respectively.
 
         Linking super/base_classes needs two data-sources, depending on whether the super/base_class is imported or not:
@@ -357,8 +350,8 @@ class ObjcRuntimeDataParser:
 
             # If the base class is an imported classref, the imported classref will be bound to its runtime load address
             # by dyld. Look up whether we have an import-binding for the `base_class` field of this structure.
-            if base_class_field_addr in dyld_info_parser.dyld_stubs_to_symbols:
-                imported_base_class_sym = dyld_info_parser.dyld_stubs_to_symbols[base_class_field_addr]
+            if base_class_field_addr in self.binary.dyld_bound_symbols:
+                imported_base_class_sym = self.binary.dyld_bound_symbols[base_class_field_addr]
                 base_class_name = imported_base_class_sym.name
 
             else:
@@ -367,7 +360,7 @@ class ObjcRuntimeDataParser:
                 if dereferenced_classref in addr_to_class_names:
                     base_class_name = addr_to_class_names[dereferenced_classref]
                 else:
-                    logging.error(
+                    logger.error(
                         f"Failed to find a corresponding ObjC class for ref {dereferenced_classref} from "
                         f"{objc_class_or_category}"
                     )
@@ -381,26 +374,27 @@ class ObjcRuntimeDataParser:
                 objc_class_or_category.superclass_name = base_class_name
 
     def _parse_global_protocol_info(self) -> List[ObjcProtocol]:
-        """Parse protocols which code in the app conforms to, referenced by __objc_protolist
-        """
-        logging.debug("Cross referencing __objc_protolist, __objc_protocol, and __objc_data entries...")
+        """Parse protocols which code in the app conforms to, referenced by __objc_protolist."""
+        logger.debug("Cross referencing __objc_protolist, __objc_protocol, and __objc_data entries...")
         protocol_pointers = self._get_protolist_pointers()
         return self._parse_protocol_ptr_list(protocol_pointers)
 
     def read_ivars_from_ivarlist_ptr(self, ivarlist_ptr: VirtualMemoryPointer) -> List[ObjcIvar]:
-        """Given the virtual address of an ivar list, return a List of each encoded ObjcIvar
-        """
+        """Given the virtual address of an ivar list, return a List of each encoded ObjcIvar."""
         ivarlist = self.binary.read_struct(ivarlist_ptr, ObjcIvarListStruct, virtual=True)
         ivars: List[ObjcIvar] = []
         # Parse each ivar struct which follows the ivarlist
         ivar_struct_ptr = ivarlist_ptr + ivarlist.sizeof
         for _ in range(ivarlist.count):
-            ivar_struct = self.binary.read_struct(ivar_struct_ptr, ObjcIvarStruct, virtual=True)
+            ivar_struct = self.binary.read_struct_with_rebased_pointers(ivar_struct_ptr, ObjcIvarStruct, virtual=True)
 
             ivar_name = self.binary.get_full_string_from_start_address(ivar_struct.name)
             class_name = self.binary.get_full_string_from_start_address(ivar_struct.type)
-            field_offset = self.binary.read_word(ivar_struct.offset_ptr, word_type=c_uint32)
             field_offset_addr = ivar_struct.offset_ptr
+            # SCAN-2960: offset_ptr may be zero. Observed for the $defaultActor ivar within a Swift source class.
+            field_offset = 0
+            if field_offset_addr:
+                field_offset = self.binary.read_word(field_offset_addr, word_type=c_uint32)
 
             # class_name and field_offset can be falsey ('' and 0), so don't include them in this sanity check
             if not ivar_name:
@@ -413,8 +407,7 @@ class ObjcRuntimeDataParser:
         return ivars
 
     def read_selectors_from_methlist_ptr(self, methlist_ptr: VirtualMemoryPointer) -> List[ObjcSelector]:
-        """Given the virtual address of a method list, return a List of ObjcSelectors encapsulating each method
-        """
+        """Given the virtual address of a method list, return a List of ObjcSelectors encapsulating each method."""
         methlist = self.binary.read_struct(methlist_ptr, ObjcMethodListStruct, virtual=True)
         selectors: List[ObjcSelector] = []
         # parse every entry in method list
@@ -438,16 +431,19 @@ class ObjcRuntimeDataParser:
 
             # save this selector in the selref pointer -> selector map
             if selref:
-                # if this selector is already in the map, check if we now know the implementation addr
+                # if this selector is already in the map, check if we now know the implementation address
                 # we could have parsed the selector literal/selref pair in _parse_selrefs() but not have known the
                 # implementation, but do now. It's also possible the selref is an external method, and thus will not
                 # have a local implementation.
+                most_specific_selector = selector
                 if selref.source_address in self._selref_ptr_to_selector_map:
                     previously_parsed_selector = self._selref_ptr_to_selector_map[selref.source_address]
-                    if not previously_parsed_selector.implementation:
-                        # delete the old entry, and add back in the next line
-                        del self._selref_ptr_to_selector_map[selref.source_address]
-                self._selref_ptr_to_selector_map[selref.source_address] = selector
+                    # Did we already parse this same selector but with more specific information?
+                    # (Say, if we parse an ObjC class implementing a protocol before parsing the protocol itself)
+                    if previously_parsed_selector.implementation:
+                        # Make sure we keep the most specific selector we've seen
+                        most_specific_selector = previously_parsed_selector
+                self._selref_ptr_to_selector_map[selref.source_address] = most_specific_selector
 
             method_entry_off += method_ent.sizeof
         return selectors
@@ -501,7 +497,7 @@ class ObjcRuntimeDataParser:
     ) -> ObjcClass:
         symbol_name = self.binary.get_full_string_from_start_address(objc_data_struct.name)
         if not symbol_name:
-            raise ValueError(f"Could not get symbol name for {objc_data_struct.name}")
+            raise ValueError(f"Could not get symbol name for {hex(objc_data_struct.name)}")
 
         selectors: List[ObjcSelector] = []
         protocols: List[ObjcProtocol] = []
@@ -520,14 +516,14 @@ class ObjcRuntimeDataParser:
         return ObjcClass(objc_class_struct, symbol_name, selectors, ivars, protocols, objc_class_struct.superclass)
 
     def _protolist_ptr_to_protocol_ptr_list(self, protolist_ptr: VirtualMemoryPointer) -> List[VirtualMemoryPointer]:
-        """Accepts the virtual address of an ObjcProtocolListStruct, and returns List of protocol pointers it refers to.
-        """
+        """Accepts the virtual address of an ObjcProtocolListStruct, and returns List of protocol pointers it refers to."""  # noqa: E501
         protolist = self.binary.read_struct(protolist_ptr, ObjcProtocolListStruct, virtual=True)
         protocol_pointers: List[VirtualMemoryPointer] = []
         # pointers start directly after the 'count' field
-        addr = protolist.binary_offset + protolist.sizeof
+        addr = VirtualMemoryPointer(protolist.binary_offset + protolist.sizeof)
         for i in range(protolist.count):
-            pointer = self.binary.read_word(addr)
+            # This pointer may be rebased
+            pointer = self.binary.read_rebased_pointer(addr)
             protocol_pointers.append(VirtualMemoryPointer(pointer))
             # step to next protocol pointer in list
             addr += sizeof(self.binary.platform_word_type)
@@ -543,41 +539,38 @@ class ObjcRuntimeDataParser:
         return protocols
 
     def _get_catlist_pointers(self) -> List[VirtualMemoryPointer]:
-        """Read pointers in __objc_catlist into list
-        """
-        _, catlist_pointers = self.binary.read_pointer_section("__objc_catlist")
-        return catlist_pointers
+        """Read pointers in __objc_catlist into list."""
+        return list(self.binary.read_pointer_section("__objc_catlist").values())
 
     def _get_protolist_pointers(self) -> List[VirtualMemoryPointer]:
-        """Read pointers in __objc_protolist into list
-        """
-        _, protolist_pointers = self.binary.read_pointer_section("__objc_protolist")
-        return protolist_pointers
+        """Read pointers in __objc_protolist into list."""
+        return list(self.binary.read_pointer_section("__objc_protolist").values())
 
     def _get_classlist_pointers(self) -> List[VirtualMemoryPointer]:
-        """Read pointers in __objc_classlist into list
-        """
-        _, classlist_pointers = self.binary.read_pointer_section("__objc_classlist")
-        return classlist_pointers
+        """Read pointers in __objc_classlist into list."""
+        return list(self.binary.read_pointer_section("__objc_classlist").values())
 
     def _get_objc_category_from_catlist_pointer(
         self, category_struct_pointer: VirtualMemoryPointer
     ) -> ObjcCategoryRawStruct:
-        """Read a struct __objc_category from the location indicated by the provided __objc_catlist pointer
-        """
-        category_entry = self.binary.read_struct(category_struct_pointer, ObjcCategoryRawStruct, virtual=True)
+        """Read a struct __objc_category from the location indicated by the provided __objc_catlist pointer."""
+        category_entry = self.binary.read_struct_with_rebased_pointers(
+            category_struct_pointer, ObjcCategoryRawStruct, virtual=True
+        )
         return category_entry
 
     def _get_objc_protocol_from_pointer(self, protocol_struct_pointer: VirtualMemoryPointer) -> ObjcProtocolRawStruct:
-        """Read a struct __objc_protocol from the location indicated by the provided struct objc_protocol_list pointer
-        """
-        protocol_entry = self.binary.read_struct(protocol_struct_pointer, ObjcProtocolRawStruct, virtual=True)
+        """Read a struct __objc_protocol from the location indicated by the provided struct objc_protocol_list pointer."""  # noqa: E501
+        protocol_entry = self.binary.read_struct_with_rebased_pointers(
+            protocol_struct_pointer, ObjcProtocolRawStruct, virtual=True
+        )
         return protocol_entry
 
     def _get_objc_class_from_classlist_pointer(self, class_struct_pointer: VirtualMemoryPointer) -> ObjcClassRawStruct:
-        """Read a struct __objc_class from the location indicated by the __objc_classlist pointer
-        """
-        class_entry = self.binary.read_struct(class_struct_pointer, ObjcClassRawStruct, virtual=True)
+        """Read a struct __objc_class from the location indicated by the __objc_classlist pointer."""
+        class_entry = self.binary.read_struct_with_rebased_pointers(
+            class_struct_pointer, ObjcClassRawStruct, virtual=True
+        )
 
         # sanitize class_entry
         # the least significant 2 bits are used for flags
@@ -590,13 +583,13 @@ class ObjcRuntimeDataParser:
         """Read a struct __objc_data from a provided struct __objc_class
         If the struct __objc_class describe invalid or no corresponding data, None will be returned.
         """
-        data_entry = self.binary.read_struct(objc_class.data, ObjcDataRawStruct, virtual=True)
+        data_entry = self.binary.read_struct_with_rebased_pointers(objc_class.data, ObjcDataRawStruct, virtual=True)
         # ensure this is a valid entry
         if data_entry.name < self.binary.get_virtual_base():
             # TODO(PT): sometimes we'll get addresses passed to this method that are actually struct __objc_method
             # entries, rather than struct __objc_data entries. Investigate why this is.
             # This was observed on a 32bit binary, Esquire2
-            logging.debug(
+            logger.debug(
                 f"caught ObjcDataRaw struct with invalid fields at {hex(int(objc_class.data))}."
                 f" data->name = {hex(data_entry.name)}"
             )
